@@ -5,6 +5,10 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 # -*- coding: utf-8 -*-
+from __future__ import annotations
+
+from abc import abstractmethod
+
 __all__ = [
     "T",
     "ArrayLike",
@@ -15,11 +19,12 @@ __all__ = [
     "OptionalNone",
 ]
 
+
 from typing import Any, Dict, TypeVar, Union
 
 import numpy as np
 import pandas as pd
-from typing_extensions import Literal, Protocol
+from typing_extensions import Protocol
 
 from dios import DictOfSeries
 
@@ -46,5 +51,14 @@ class GenericFunction(Protocol):
         ...  # pragma: no cover
 
 
+class Comparable(Protocol):
+    @abstractmethod
+    def __gt__(self: CompT, other: CompT) -> bool:
+        pass
+
+
+CompT = TypeVar("CompT", bound=Comparable)
+
+
 class OptionalNone:
     pass
