@@ -13,15 +13,15 @@ import pandas as pd
 import pytest
 
 import saqc
-from saqc.core import SaQC, initFlagsLike, register
+from saqc.constants import BAD, FILTER_ALL, FILTER_NONE, UNFLAGGED
+from saqc.core import SaQC, initFlagsLike
 from saqc.core.flags import Flags
-from saqc.core.register import flagging, processing
-from tests.common import flagAll, initData
+from saqc.core.register import flagging, processing, register
+from saqc.lib.types import OptionalNone
+from tests.common import initData
 
 OPTIONAL = [False, True]
 
-register(mask=["field"], demask=["field"], squeeze=["field"])(flagAll)
-
 
 @pytest.fixture
 def data():
@@ -36,7 +36,7 @@ def flags(data, optional):
 
 def test_errorHandling(data):
     @processing()
-    def raisingFunc(data, field, flags, **kwargs):
+    def raisingFunc(saqc, field, **kwargs):
         raise TypeError
 
     var1 = data.columns[0]
@@ -51,6 +51,12 @@ def test_dtypes(data, flags):
     """
     Test if the categorical dtype is preserved through the core functionality
     """
+
+    @register(mask=["field"], demask=["field"], squeeze=["field"])
+    def flagAll(saqc, field, **kwargs):
+        saqc._flags[:, field] = BAD
+        return saqc
+
     flags = initFlagsLike(data)
     flags_raw = flags.toDios()
     var1, var2 = data.columns[:2]
@@ -61,6 +67,11 @@ def test_dtypes(data, flags):
         assert pflags[c].dtype == flags[c].dtype
 
 
+def test_new_call(data):
+    qc = saqc.SaQC(data)
+    qc = qc.flagRange("var1", max=5)
+
+
 def test_copy(data):
     qc = saqc.SaQC(data)
 
@@ -100,9 +111,9 @@ def test_sourceTargetCopy():
     target = "new"
 
     @register(mask=["field"], demask=["field"], squeeze=["field"], handles_target=False)
-    def flagTarget(data, field, flags, **kwargs):
+    def flagTarget(saqc, field, **kwargs):
         assert "target" not in kwargs
-        return data, flags
+        return saqc
 
     qc = SaQC(data, flags=Flags({var1: pd.Series(127.0, index=data[var1].index)}))
     qc = qc.flagTarget(field=var1, target=target)
@@ -120,18 +131,18 @@ def test_sourceTargetNoCopy():
     target = "new"
 
     @register(mask=["field"], demask=["field"], squeeze=["field"], handles_target=True)
-    def flagField(data, field, flags, **kwargs):
+    def flagField(saqc, field, **kwargs):
         assert "target" in kwargs
-        assert "target" not in data
-        assert "target" not in flags
-        return data, flags
+        assert "target" not in saqc._data
+        assert "target" not in saqc._flags
+        return saqc
 
     SaQC(data).flagField(field=var1, target=target)
 
 
 def test_sourceTargetMultivariate():
     """
-    test bypassing of the imlpicit copy machiners
+    test bypassing of the imlpicit copy machinery
     """
     data = initData(3)
 
@@ -142,12 +153,12 @@ def test_sourceTargetMultivariate():
         handles_target=True,
         multivariate=True,
     )
-    def flagMulti(data, field, flags, **kwargs):
+    def flagMulti(saqc, field, **kwargs):
         assert "target" in kwargs
-        assert "target" not in data
-        assert "target" not in flags
+        assert "target" not in saqc._data
+        assert "target" not in saqc._flags
         assert field == kwargs["target"]
-        return data, flags
+        return saqc
 
     SaQC(data).flagMulti(field=data.columns, target=data.columns)
 
@@ -162,20 +173,17 @@ def test_sourceTargetMulti():
         mask=["field"],
         demask=["field"],
         squeeze=["field"],
-        handles_target=False,
+        handles_target=True,
         multivariate=True,
     )
-    def flagMulti(data, field, flags, target, **kwargs):
+    def flagMulti(saqc, field, target, **kwargs):
         assert len(field) == len(target)
         for src, trg in zip(field, target):
-            assert src in data
-            assert trg in data
-            assert src in flags
-            assert trg in flags
-
-            assert (data[src] == data[trg]).all(axis=None)
-            assert (flags[src] == flags[trg]).all(axis=None)
-        return data, flags
+            assert src in saqc._data
+            assert src in saqc._flags
+            assert trg not in saqc._data
+            assert trg not in saqc._flags
+        return saqc
 
     SaQC(data, flags).flagMulti(field=fields, target=targets)
 
@@ -196,28 +204,9 @@ def test_validation(data):
     qc = SaQC(df)
 
     @flagging()
-    def flagFoo(data, field, flags, **kwargs):
-        data["spam"] = data[field]
-        return data, flags
-
-    with pytest.raises(RuntimeError):
-        qc.flagFoo("a")
-
-
-@pytest.mark.skip(reason="bug in register, see #GL 342")
-def test_validation_flags(data):
-    """Test if validation detects different columns in data and flags."""
-    df = pd.DataFrame(
-        data=np.arange(8).reshape(4, 2),
-        index=pd.date_range("2020", None, 4, "1d"),
-        columns=list("ab"),
-    )
-    qc = SaQC(df)
-
-    @flagging()
-    def flagFoo(data, field, flags, **kwargs):
-        flags["spam"] = flags[field]
-        return data, flags
+    def flagFoo(saqc, field, **kwargs):
+        saqc._data["spam"] = saqc._data[field]
+        return saqc
 
     with pytest.raises(RuntimeError):
         qc.flagFoo("a")
@@ -245,3 +234,136 @@ def test_immutability(data):
     saqc_after = saqc_before.flagDummy(field)
     for name in SaQC._attributes:
         assert getattr(saqc_before, name) is not getattr(saqc_after, name)
+
+
+@pytest.mark.parametrize(
+    "field,target",
+    [
+        (["a"], ["x", "y"]),
+        (["a", "b"], ["x"]),
+    ],
+)
+def test_fieldsTargetsExpansionFail(field, target):
+    # check that the field/target handling works as expected for the
+    # different function types
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=False, handles_target=False)
+    def foo(saqc, field, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=False, handles_target=True)
+    def bar(saqc, field, target, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=True, handles_target=False)
+    def baz(saqc, field, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=False, handles_target=True)
+    def fooBar(saqc, field, **kwargs):
+        return saqc
+
+    data = pd.DataFrame({"a": [1, 2], "b": [2, 3], "c": [3, 4], "d": [4, 5]})
+    qc = SaQC(data)
+    with pytest.raises(ValueError):
+        qc.foo(field, target=target)
+    with pytest.raises(ValueError):
+        qc.bar(field, target=target)
+    with pytest.raises(ValueError):
+        qc.baz(field, target=target)
+    with pytest.raises(ValueError):
+        qc.fooBar(field, target=target)
+
+
+@pytest.mark.parametrize(
+    "field,target",
+    [
+        (["a"], ["x"]),
+        (["a", "a"], ["x", "y"]),
+    ],
+)
+def test_fieldsTargetsExpansion(field, target):
+    # check that the field/target handling works as expected for the
+    # different function types
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=False, handles_target=False)
+    def foo(saqc, field, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=False, handles_target=True)
+    def bar(saqc, field, target, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=True, handles_target=False)
+    def baz(saqc, field, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=False, handles_target=True)
+    def fooBar(saqc, field, **kwargs):
+        return saqc
+
+    data = pd.DataFrame({"a": [1, 2], "b": [2, 3], "c": [3, 4], "d": [4, 5]})
+    qc = SaQC(data)
+    qc.foo(field, target=target)
+    qc.bar(field, target=target)
+    qc.baz(field, target=target)
+    qc.fooBar(field, target=target)
+
+
+@pytest.mark.parametrize(
+    "field,target",
+    [
+        (["a"], ["x"]),
+        (["a", "a"], ["x", "y"]),
+        (["a"], ["x", "y"]),
+        (["a", "a"], ["x"]),
+    ],
+)
+def test_fieldsTargetsExpansionMultivariate(field, target):
+    @register(mask=[], demask=[], squeeze=[], multivariate=True, handles_target=True)
+    def foo(saqc, field, target, **kwargs):
+        return saqc
+
+    @register(mask=[], demask=[], squeeze=[], multivariate=True, handles_target=True)
+    def bar(saqc, field, **kwargs):
+        return saqc
+
+    data = pd.DataFrame(
+        {"a": [1, 2, 3], "b": [2, 3, 4], "c": [3, 4, 5], "d": [4, 5, 6]}
+    )
+    qc = SaQC(data)
+    qc.foo(field, target)
+    qc.bar(field, target)
+
+
+def test_columnConsitency(data):
+    @flagging()
+    def flagFoo(saqc, field, **kwargs):
+        saqc._flags["spam"] = saqc._flags[field]
+        return saqc
+
+    field = data.columns[0]
+    qc = SaQC(data)
+    with pytest.raises(RuntimeError):
+        qc.flagFoo(field)
+
+
+@pytest.mark.parametrize(
+    "user_flag,internal_flag",
+    (
+        [FILTER_ALL, FILTER_ALL],
+        [FILTER_NONE, FILTER_NONE],
+        [OptionalNone(), FILTER_ALL],
+        ["BAD", BAD],
+        ["UNFLAGGED", UNFLAGGED],
+    ),
+)
+def test_dfilterTranslation(data, user_flag, internal_flag):
+    @flagging()
+    def flagFoo(saqc, field, dfilter, **kwargs):
+        assert dfilter == internal_flag
+        return saqc
+
+    field = data.columns[0]
+    qc = SaQC(data, scheme="simple")
+    qc.flagFoo(field, dfilter=user_flag)
