@@ -10,8 +10,7 @@ import numpy as np
 import pytest
 
 from saqc.constants import BAD, UNFLAGGED
-from saqc.core import initFlagsLike
-from saqc.funcs.constants import flagByVariance, flagConstants
+from saqc.core import SaQC, initFlagsLike
 from tests.common import initData
 
 
@@ -27,10 +26,9 @@ def data():
 def test_constants_flagBasic(data):
     field, *_ = data.columns
     flags = initFlagsLike(data)
-    data, flags_result = flagConstants(
-        data, field, flags, window="15Min", thresh=0.1, flag=BAD
-    )
-    flagscol = flags_result[field]
+    qc = SaQC(data, flags)
+    qc = qc.flagConstants(field, window="15Min", thresh=0.1, flag=BAD)
+    flagscol = qc._flags[field]
     assert np.all(flagscol[5:25] == BAD)
     assert np.all(flagscol[:5] == UNFLAGGED)
     assert np.all(flagscol[25 + 1 :] == UNFLAGGED)
@@ -39,11 +37,10 @@ def test_constants_flagBasic(data):
 def test_constants_flagVarianceBased(data):
     field, *_ = data.columns
     flags = initFlagsLike(data)
-    data, flags_result1 = flagByVariance(
-        data, field, flags, window="1h", thresh=0.0005, flag=BAD
-    )
+    qc = SaQC(data, flags)
+    qc = qc.flagByVariance(field, window="1h", thresh=0.0005, flag=BAD)
 
-    flagscol = flags_result1[field]
+    flagscol = qc._flags[field]
     assert np.all(flagscol[5:25] == BAD)
     assert np.all(flagscol[:5] == UNFLAGGED)
     assert np.all(flagscol[25 + 1 :] == UNFLAGGED)
